@@ -29,13 +29,14 @@ cb_user_identity_free (CbUserIdentity *id)
 }
 
 void
-cb_user_identity_copy (CbUserIdentity *id, CbUserIdentity *id2)
+cb_user_identity_copy (const CbUserIdentity *id, CbUserIdentity *id2)
 {
   g_free (id2->screen_name);
   id2->screen_name = g_strdup (id->screen_name);
   g_free (id2->user_name);
   id2->user_name = g_strdup (id->user_name);
   id2->id = id->id;
+  id2->verified = id->verified;
 }
 
 void cb_user_identity_parse (CbUserIdentity *id,
@@ -44,6 +45,7 @@ void cb_user_identity_parse (CbUserIdentity *id,
   id->id = json_object_get_int_member (user_obj, "id");
   id->screen_name = g_strdup (json_object_get_string_member (user_obj, "screen_name"));
   id->user_name = cb_utils_escape_ampersands (json_object_get_string_member (user_obj, "name"));
+  id->verified = json_object_get_boolean_member (user_obj, "verified");
 }
 
 
@@ -56,7 +58,7 @@ cb_text_entity_free (CbTextEntity *e)
 }
 
 void
-cb_text_entity_copy (CbTextEntity *e1, CbTextEntity *e2)
+cb_text_entity_copy (const CbTextEntity *e1, CbTextEntity *e2)
 {
   e2->from = e1->from;
   e2->to   = e1->to;
@@ -94,13 +96,12 @@ cb_mini_tweet_free (CbMiniTweet *t)
 }
 
 void
-cb_mini_tweet_copy (CbMiniTweet *t1, CbMiniTweet *t2)
+cb_mini_tweet_copy (const CbMiniTweet *t1, CbMiniTweet *t2)
 {
   guint i;
 
   t2->id = t1->id;
   t2->created_at = t1->created_at;
-  cb_user_identity_free (&t2->author);
   cb_user_identity_copy (&t1->author, &t2->author);
   g_free (t2->text);
   t2->text = g_strdup (t1->text);
@@ -394,15 +395,20 @@ cb_mini_tweet_parse_entities (CbMiniTweet *t,
   /* entities->media and extended_entities contain exactly the same media objects,
      but extended_entities is not always present, and entities->media doesn't
      contain all the attached media, so parse both the same way... */
-
-  i = 0;
-  if (json_object_has_member (entities, "media")) n_media_arrays ++;
-  if (json_object_has_member (status, "extended_entities")) n_media_arrays ++;
   if (json_object_has_member (entities, "media"))
-    media_arrays[i++] = json_object_get_array_member (entities, "media");
+    {
+      media_arrays[n_media_arrays] = json_object_get_array_member (entities, "media");
+      n_media_arrays ++;
+    }
+
   if (json_object_has_member (status, "extended_entities"))
-    media_arrays[i] = json_object_get_array_member (json_object_get_object_member (status, "extended_entities"),
-                                                    "media");
+    {
+      media_arrays[n_media_arrays] = json_object_get_array_member (json_object_get_object_member (status,
+                                                                                                  "extended_entities"),
+                                                                   "media");
+
+      n_media_arrays ++;
+    }
 
   for (i = 0; i < n_media_arrays; i ++)
     {
@@ -491,7 +497,8 @@ cb_mini_tweet_parse_entities (CbMiniTweet *t,
                    */
                   for (k = 0; k < t->n_medias; k ++)
                     {
-                      if (t->medias[k]->type == CB_MEDIA_TYPE_IMAGE &&
+                      if (t->medias[k] != NULL &&
+                          t->medias[k]->type == CB_MEDIA_TYPE_IMAGE &&
                           strcmp (t->medias[k]->url, thumb_url) == 0)
                         {
                           /* Replace this media */
