@@ -35,6 +35,13 @@ token_str (const Token *t)
 {
   return g_strdup_printf ("Type: %u, Text: '%.*s'", t->type, (int)t->length_in_bytes, t->start);
 }
+
+static char * G_GNUC_UNUSED
+entity_str (const TlEntity *e)
+{
+  return g_strdup_printf ("Type: %u, Text: '%.*s'", e->type, (int)e->length_in_bytes, e->start);
+}
+
 #endif
 
 enum {
@@ -119,6 +126,33 @@ token_type_from_char (gunichar c)
   }
 }
 
+static inline gboolean
+token_ends_in_accented (const Token *t)
+{
+  const char *p = t->start;
+  gunichar c;
+  gsize i;
+
+  if (t->length_in_bytes == 1 ||
+      t->type != TOK_TEXT) {
+    return FALSE;
+  }
+
+  // The rules here aren't exactly clear...
+  // We read the last character of the text pointed to by the given token.
+  // If that's not an ascii character, we return TRUE.
+  for (i = 0; i < t->length_in_characters - 1; i ++) {
+    p = g_utf8_next_char (p);
+  }
+
+  c = g_utf8_get_char (p);
+
+  if (c > 127)
+    return TRUE;
+
+  return FALSE;
+}
+
 static inline gboolean
 token_in (const Token *t,
           const char  *haystack)
@@ -185,6 +219,17 @@ emplace_entity_for_tokens (GArray      *array,
   }
 }
 
+static inline gboolean
+is_valid_mention_char (gunichar c)
+{
+  // Just ASCII
+  if (c > 127)
+    return FALSE;
+
+  return TRUE;
+}
+
+
 static inline gboolean
 token_is_tld (const Token *t,
               gboolean     has_protocol)
@@ -574,7 +619,8 @@ parse_mention (GArray      *entities,
     // Text tokens before an @-token generally destroy the mention,
     // except in a few cases...
     if (tokens[i - 1].type == TOK_TEXT &&
-        !token_in (&tokens[i - 1], VALID_BEFORE_MENTION_CHARS)) {
+        !token_in (&tokens[i - 1], VALID_BEFORE_MENTION_CHARS) &&
+        !token_ends_in_accented (&tokens[i - 1])) {
       return FALSE;
     }
 
@@ -606,6 +652,23 @@ parse_mention (GArray      *entities,
       break;
     }
 
+    if (tokens[i].type == TOK_TEXT) {
+      const char *text = tokens[i].start;
+      // Special rules apply about what characters may appear in a @screen_name
+      const char *p = text;
+
+      while (p - text < (long)tokens[i].length_in_bytes) {
+        gunichar c = g_utf8_get_char (p);
+
+        if (!is_valid_mention_char (c)) {
+          return FALSE;
+        }
+
+        p = g_utf8_next_char (p);
+      }
+
+    }
+
     i ++;
   }
 
@@ -648,7 +711,13 @@ parse_hashtag (GArray      *entities,
 
   // Lookback at the previous token. If it was a text token
   // without whitespace between, this is not going to be a mention...
-  if (i > 0 && tokens[i - 1].type == TOK_TEXT) {
+  if (i > 0 && tokens[i - 1].type == TOK_TEXT &&
+      !token_in (&tokens[i - 1], VALID_BEFORE_HASHTAG_CHARS)) {
+    return FALSE;
+  }
+
+  // Some chars make the entire hashtag invalid
+  if (i > 0 && token_in (&tokens[i - 1], INVALID_BEFORE_HASHTAG_CHARS)) {
     return FALSE;
   }
 
@@ -727,13 +796,14 @@ parse (const Token *tokens,
       break;
     }
 
-    if (extract_text_entities) {
+    if (extract_text_entities &&
+        token->type == TOK_TEXT) {
       relevant_entities ++;
     }
 
     emplace_entity_for_tokens (entities,
                                tokens,
-                               token->type == TOK_WHITESPACE ? TL_ENT_WHITESPACE : TL_ENT_TEXT,
+                               token->type == TOK_TEXT ? TL_ENT_TEXT : TL_ENT_WHITESPACE,
                                i, i);
 
     i ++;
@@ -763,8 +833,9 @@ count_entities_in_characters (GArray *entities)
 
 /*
  * tl_count_chars:
- * input: (nullable): Tweet text
+ * input: (nullable): NUL-terminated tweet text
  *
+ * Returns: The length of @input, in characters.
  */
 gsize
 tl_count_characters (const char *input)
@@ -781,7 +852,7 @@ tl_count_characters (const char *input)
  * input: (nullable): Text to measure
  * length_in_bytes: Length of @input, in bytes.
  *
- * TODO: We might want to do a g_utf8_make_valid or at least _validate.
+ * Returns: The length of @input, in characters.
  */
 gsize
 tl_count_characters_n (const char *input,
@@ -887,7 +958,7 @@ tl_extract_entities_internal (const char *input,
           memcpy (&result_entities[result_index], e, sizeof (TlEntity));
           result_index ++;
         }
-        break;
+      break;
 
       default: {}
     }
